#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import logger
from config import *

def pvcreate (pv):
    logger.run_command (PVCREATE_PATH + " -ff -y " + pv)

def vgcreate (vg, pv):
    logger.run_command (VGCREATE_PATH + " " + vg + " " + pv)

def vgchange (vg, active):
    flag = "y"
    if not active:
        flag = "n"
    logger.run_command (VGCHANGE_PATH + " -a" + flag + " " + vg)

def lvcreate (vg, lv, size, readonly = False, origin = None):
    size_in_kb = (size + 1023) / 1024
    
    if readonly:
        perms = "r"
    else:
        perms = "rw"
    
    if origin is None:
        logger.run_command (LVCREATE_PATH + " --size %sk -p %s --name %s %s" % (size_in_kb, perms, lv, vg))
    else:
        logger.run_command (LVCREATE_PATH + " -s --size %sk -p %s --name %s %s" % (size_in_kb, perms, lv, origin))

def lvconvert_to_snapshot (origin, cow, zero = True):
    zero_org = ""
    if not zero:
        zero_arg = "-Zn"
    
    logger.run_command (LVCONVERT_PATH + " %s -s %s %s" % (zero_arg, origin, cow))

def lvmerge (snapshot):
    logger.run_command (LVMERGE_PATH + " -f " + snapshot)

def lvrename (old, new):
    logger.run_command (LVRENAME_PATH + " " + old + " " + new)

def deactivate_vgs_on_dev (device):
    output = logger.run_command (PVS_PATH + " --noheadings -o pv_name,vg_name")

    for line in output.split ("\n"):
        parts = line.split ()
        if len (parts) != 2:
            continue

        if not parts[0].startswith (device):
            continue

        vgchange (parts[1], False)

def get_pvs_from_vg (vg):
    output = logger.run_command (PVS_PATH + " --noheadings -o vg_name,pv_name")

    pvs = []

    for line in output.split ("\n"):
        parts = line.split ()
        if len (parts) != 2:
            continue

        if parts[0] != vg:
            continue

        pvs.append (parts[1])

    return pvs

class VolumeGroupInfo:
    """Encapsulates the names and paths of the volumes we
    use in our volume group.

    The following members are available:

      + @lvmdev: the physical volume which contains @volumegroup
      + @volumegroup: the VG containing the LVs below
      + @rootlv: the readonly snapshot which the client boots from
      + @swaplv: our swap volume
      + @originlv: the origin of the root snapshot to which updates
                   are merged
      + @updatelv: the volume to which updates are downloaded to
                   before merging

    Each of the above LVs has a corresponding @lv_path member
    which contains the full path to the volume.
    """
    
    def __init__ (self, lvmdev, volumegroup, rootlv, swaplv):
        """Initialize a %VolumeGroupInfo using the given
        volume group, root volume and swap volume names.
        """
        self.lvmdev = lvmdev
        
        self.volumegroup = volumegroup
        self.rootlv = rootlv
        self.swaplv = swaplv

        self.originlv = rootlv + "_origin"
        self.updatelv = rootlv + "_update"

        self.rootlv_path = self._build_lv_path (self.rootlv)
        self.swaplv_path = self._build_lv_path (self.swaplv)
        self.originlv_path = self._build_lv_path (self.originlv)
        self.updatelv_path = self._build_lv_path ( self.updatelv)
        
    def _build_lv_path (self, lv):
        return os.path.join ("/dev", self.volumegroup, lv)
