#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os.path
import exceptions
import tempfile
import parted
from config import *

from stacaccli.common import bootloader
from stacaccli.common import grabutils
from stacaccli.common import imagemetadata
from stacaccli.common import logger
from stacaccli.common import lvm
from stacaccli.common import shvar
from stacaccli.common import sysutils
from stacaccli.common.config import *

class UpdaterError (exceptions.Exception):
    def __init__ (self, args = None):
        exceptions.Exception.__init__ (self)
        self.args = args

class Update:
    def __init__ (self, repo, image_name):
        self._repo = repo
        self._image_name = image_name

        imagemetadata.clear_cache (CACHE_DIR)

    def _find_bootdev (self, device):
        def read_e2label (path):
            return logger.run_command (E2LABEL_PATH + " " + path)
        
        disk = parted.PedDisk.new (parted.PedDevice.get (device))
        
        part = disk.next_partition ()
        while not part is None:
            if part.fs_type and part.fs_type.name == "ext3":
                path = sysutils.get_partition_path (part)
                if read_e2label (path) == "/boot":
                    return sysutils.get_partition_path (part)

            part = disk.next_partition (part)

        raise UpdaterError (_("Cannot find ext3 partition on '%s' with /boot label") % device)

    def _find_devices (self, image):
        pvs = lvm.get_pvs_from_vg (image.volumegroup)
        
        if not pvs:
            raise UpdaterError (_("Cannot find physical volume for volume group '%s'") %
                                image.volumegroup)
        if len (pvs) > 1:
            raise UpdaterError (_("More than one physical volume for volume group '%s'") %
                                image.volumegroup)

        lvmdev = pvs[0]

        device = sysutils.get_partition_parent_path (lvmdev)

        bootdev = self._find_bootdev (device)

        return (device, lvmdev, bootdev)

    def _get_current_revision (self, vgi):
        tmpdir = tempfile.mkdtemp (dir = "/tmp", prefix = "root-")

        logger.run_command (MOUNT_PATH + " -o ro " +
                            vgi.originlv_path + " " + tmpdir)

        config_path = os.path.join (tmpdir, CONFIG_PATH[1:])
        current_revision = shvar.Config(config_path).image_revision

        logger.run_command (UMOUNT_PATH + " " + tmpdir)
        os.rmdir (tmpdir)

        if current_revision is None:
            raise UpdateError (_("No IMAGE_REVISION found in %s") % CONFIG_PATH)
        
        try:
            current_revision = int (current_revision)
        except ValueError:
            raise UpdateError (_("IMAGE_REVISION in %s should be an integer, not '%s'") %
                               (CONFIG_PATH, current_revision))

        return current_revision

    def _download_update (self, image_name, revision, size, vgi):
        update_url = self._repo + "/" + image_name + "-" + str (revision) + ".cow"
        
        grabutils.urlgrab_to_dev (update_url, size, vgi.updatelv_path)

    def _merge_update (self, vgi, image_size):
        # Convert updatelv into a snapshot of originlv
        lvm.lvconvert_to_snapshot (vgi.originlv_path, vgi.updatelv_path, False)
        
        # Merge the snapshot into its origin
        lvm.lvmerge (vgi.updatelv_path)

        # Re-name the merged snapshot as originlv
        lvm.lvrename (vgi.updatelv_path, vgi.originlv_path)

        # Re-create updatelv
        lvm.lvcreate (vgi.volumegroup, vgi.updatelv, image_size)

    def run (self):
        image = imagemetadata.get_image (self._repo,
                                         self._image_name,
                                         CACHE_DIR)
        if image is None:
            logger.verbose ("No change in metadata for '%s' at '%s'" %
                            (self._image_name, self._repo))
            return

        (device, lvmdev, bootdev) = self._find_devices (image)

        vgi = lvm.VolumeGroupInfo (lvmdev,
                                   image.volumegroup,
                                   image.rootlv,
                                   image.swaplv)

        current_revision = self._get_current_revision (vgi)
        current_kernelversion = image.kernelversion

        for (rev_id, rev_size, rev_kernelversion) in image.revisions:
            if current_revision >= rev_id:
                logger.verbose (("Not updating image '%s' to revision %d;" +
                                 "we're at %d") %
                                (image.name, rev_id, current_revision))
                current_kernelversion = rev_kernelversion
                continue
               
            logger.verbose ("Updating image '%s' to revision %d" %
                            (image.name, rev_id))

            self._download_update (image.name, rev_id, rev_size, vgi)
            self._merge_update (vgi, image.size)

            if current_kernelversion != rev_kernelversion:
                bootloader.update_bootloader (image,
                                              device,
                                              bootdev,
                                              vgi,
                                              self._repo,
                                              rev_kernelversion)

            current_kernelversion = rev_kernelversion

    def cleanup (self):
        imagemetadata.clear_cache (CACHE_DIR)
